%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matlab M-file       Project: HyEQ Toolbox  @ Hybrid Dynamics and Control
% Lab, http://www.u.arizona.edu/~sricardo/index.php?n=Main.Software
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% HyEQSimbolicSolver solves hybrid equations (simbolically)
%   [t j x] = HyEQSimbolicSolver(f,g,C,D,x0,TSPAN,JSPAN) computes solutions
%   to the hybrid equations
%
%   \dot{x} = f(x)  x \in C x^+ = g(x)  x \in D
%
%   where x is the state, f is the flow map, g is the jump map, C is the
%   flow set, and D is the jump set. It outputs the state trajectory (t,j)
%   -> x(t,j), where t is the flow time parameter and j is the jump
%   parameter.
%
%   x0 defines the initial condition for the state.
%
%   TSPAN = [TSTART TFINAL] is the time interval. JSPAN = [JSTART JSTOP] is
%       the interval for discrete jumps. The algorithm stop when the first
%       stop condition is reached.
%
%   rule for jumps
%       rule = 1 (default) -> priority for jumps rule = 2 -> priority for
%       flows
%
%   (vpaPr) Variable precision arithmetic
%       vpa see 'help vpa' vpaPr default value = 32
%  

%% Simulation variables initialization
function [t j x] = HyEQSimbolicSolver(f,g,C,D,x0,TSPAN,JSPAN,xsym,tsym,rule,vpaPrin)
if ~exist('rule','var')
    rule = 1; % Default value
end
global vpaPr % vpa Variable precision arithmetic 
if ~exist('vpaPrin','var')
    vpaPr = 32; % Default value
else
    vpaPr = vpaPrin;
end

% simulation horizon
tstart = TSPAN(1);
tfinal = TSPAN(end);

% simulate
tout(1,1) = sym(tstart);
tout(1,2) = sym(tstart);
xout{1} = symfun(x0.',tsym);
jout = JSPAN(1);
j = jout(end);

% set symbolic equations for flow and jump sets
Ceq = C(xsym,tsym);
Deq = D(xsym,tsym);
Cbar = borderEq(Ceq,xsym,tsym);
Dbar = borderEq(Deq,xsym,tsym);
Cbar_xi = Cbar;
Dbar_xi = Dbar;

% set the symbolic varialbes from xsym
xt = xsym(tsym); % get the indexed sym
Xi = sym('xi',[length(xt),1]);
Xisym = symfun(Xi,tsym);
syms xi0

for ixt = 1:length(xt)
    syms(char(xt(ixt)));
    syms(char(Xi(ixt)));
end

for ixi = 1:length(xt)
    Cbar_xi = subs(Cbar_xi,xt(ixi),Xi(ixi));
    Dbar_xi = subs(Dbar_xi,xt(ixi),Xi(ixi));
end

[rCeq,cCeq] = size(Ceq);
[rDeq,cDeq] = size(Deq);
for irCeq=1:rCeq % get the symbolic variables in each set equation
    for icCeq=1:cCeq
        varCbar_xi{irCeq,icCeq} = symvar(Cbar_xi(irCeq,icCeq));
    end 
end
for irDeq=1:rDeq
    for icDeq=1:cDeq
        varDbar_xi{irDeq,icDeq} = symvar(Dbar_xi(irDeq,icDeq));
    end 
end

%% Jump if jump is prioritized:
if rule == 1
    while (j<JSPAN(end))
        % Check if value it is possible to jump current position
        [rDeq,cDeq] = size(Deq);
        insideD = 0;
        for irDeq = 1:rDeq % check for all disconnected sets
            iDeq = Deq(irDeq,:);
            ttemp = tout(end,2);
            insideD = isinSet(iDeq,xout{end}(ttemp).',xsym,tsym);            
            if insideD
                break
            end
        end
        if insideD == 1
            [j tout jout xout] = jump(g,j,tout,jout,xout,tsym);
        else
            break;
        end
    end
end
fprintf('Completed: %3.0f%% thd [time: %3.5f, jumps: %3.0f ]\n',...
        0,0,0);

while (j < JSPAN(end) && eval(subs(tout(end,2) < TSPAN(end))))
    %% Variables initialization
    IjC = {}; IjLeftC = {}; IjRightC = {}; 
    IjD = {}; IjLeftD = {}; IjRightD = {}; 
    IjEqC = {}; IjEqD = {};
    irCeq_Ij = [];
    irDeq_Ij = [];
    xsol = [];
    tini = tout(end,2); % Initial time to solve equations
    
    %% Solve the flow equations at current position (subfunction)
    %     ttemp = vpa(tout(end,1),vpaPr);
    ttemp = tout(end,2);
    xsol = dsolve(diff(xsym) == f(xt),...
        xsym(ttemp)==(xout{end}(ttemp).'),'IgnoreAnalyticConstraints', true);
    if isempty(xsol)
        break
    end
    xfields = fieldnames(xsol); % get the 'names' of the variables
    %-------------
    
    % Check if it is possible to flow from current position
    insideC = 0;
    for irCeq = 1:rCeq % check for all C disconnected sets   
        iCeq = Ceq(irCeq,:);
        ttemp = tout(end,2);
        insideC = isinSet(iCeq,xout{end}(ttemp).',xsym,tsym);
        if insideC == 1
            % Solve the interval for this set
            % for all C disconnected sets
            try
                [IjC{end+1},IjLeftC{end+1},IjRightC{end+1},IjEqC{end+1}] = ...
                    zeroevents(iCeq,xsol,xt,tsym,tini);
                irCeq_Ij(end+1) = irCeq;
            catch
            end
        end
    end
    switch rule % flow or jump priority
        case 1 % jump priority (solve xsol, x\in C, x\in\close(Rn\setminus D))
            for irDeq = 1:rDeq % check for all D disconnected sets
                % Solve the event t for this set
                    iDeq = Deq(irDeq,:);
                    try
                    [IjD{end+1},IjLeftD{end+1},IjRightD{end+1},IjEqD{end+1}] = ...
                        zeroevents(iDeq,xsol,xt,tsym,tini);
                    irDeq_Ij(end+1) = irDeq;
                    catch
                    end
            end
        case 2 % flow priority (solve xsol, x\in C)
            %         if (eval(sym(sum(IjLeft==tout(end,2))))==length(IjLeft))&&~isempty(Ij)
            
    end
    %% Update the ouput and state vectors for flows
    if ~isempty(IjC) % For the case that no flow is possible
        if isempty(IjD)
            oldrule = rule;
            rule = 2;
        end
        %--- Flow function
        tout(end+1,1) = tout(end,2);
        if rule==1% jump priority (solve xsol, x\in C, x\in\close(Rn\setminus D))
            [RvalC ivalC] = min(eval(subs(IjRightC)));
            [RvalD ivalD] = min(eval(subs(IjLeftD)));
            [Rval ival] = min([RvalD,RvalC]);
            if ival==1 % minimum is in LvalD
                tout(end,2) = IjLeftD{ivalD};
            elseif  ival==2% minimum is in RvalC
                tout(end,2) = IjRightC{ivalC};                
            end                                        
        elseif rule==2% flow priority (solve xsol, x\in C)
            ival=2;
            [RvalC ivalC] = max(eval(subs(IjRightC)));
            tout(end,2) = IjRightC{ivalC};
        end
        %--- Flow function
        %--- End point of the flow part
        tout(end+1,1) = tout(end,2);
        if rule==1% jump priority (solve xsol, x\in C, x\in\close(Rn\setminus D))
            if ival==1 % minimum is in LvalD
                tout(end,2) = IjLeftD{ivalD};
            else % minimum is in RvalC
                tout(end,2) = IjRightC{ivalC};                
            end                                        
        elseif rule==2% flow priority (solve xsol, x\in C)
            tout(end,2) = IjRightC{ivalC};
        end
        %--- End point of the flow part
        if isempty(IjD)
            rule = oldrule;
        end
        % Get solution function (from symbolic solution)
        %--- Flow function
        clear xouttemp;
        for ixt = 1:length(xt)
            str = regexprep(char(xt(ixt)),['\(',char(tsym),'\)'],'');
            for iixt = 1:length(xt)
                if strcmp(xfields{iixt},str);
                    xouttemp(ixt) = getfield(xsol, xfields{iixt});
                    break
                end
            end
        end
        xout{end+1} = symfun(xouttemp,tsym); 
        jout(end+1) = j;
        %--- End point of the flow part
        jout(end+1) = j;      
        ttemp = tout(end,2);        
        xstar = xout{end}(ttemp);
        Si = sym([]);
        varSi = sym([]);
        if ival==1 % flow ends up to solution reaches D 
            S = Dbar_xi(irDeq_Ij(ivalD),:);            
            varS = [varDbar_xi{irDeq_Ij(ivalD),:}];
        elseif  ival==2% % flow ends up to solution leaves C
            S = Cbar_xi(irCeq_Ij(ivalC),:);            
            varS = [varCbar_xi{irCeq_Ij(ivalC),:}];                        
        end
        for iSi = 1:length(S)
            if isinSet(S(iSi),xstar,Xisym,tsym);
                Si(end+1) = S(iSi);
                varSi = union(varSi,varS(iSi));
            end
        end
        try
%             xistar = solve(eval(Si),xi0==0,xi0,varSi); % xi0 equation is added to guaranty xistar as structure.\ BUG!
            xistar = solve(eval(Si),xi0==0); % xi0 equation is added to guaranty xistar as structure.\
        catch
            warning('HyEQsymb:SetEquationsNotSatsf','During symbolic solution: Equation set at flow "end point" not satisfied.')
        end
        xistarfields = fieldnames(xistar); % get the 'names' of the variables

        for ixt = 1:length(Xi)
            for iixt = 1:length(xistarfields)
                if strcmp(xistarfields{iixt},char(Xi(ixt)));
                    xstar(ixt) = getfield(xistar, xistarfields{iixt});
                    break
                end
            end
        end
        
        xout{end+1} = symfun(xstar,tsym);
        
    end
    %% Check for jumps and if x+\in C or x+\in D
    switch rule % flow or jump priority
        case 1 % jump priority (solve as much jumps as possible)
            while (j<JSPAN(end))
                % Check if it is possible to jump from current position
                insideD = 0;
                for irDeq = 1:rDeq % check for all disconnected sets
                    iDeq = Deq(irDeq,:);
                    ttemp = tout(end,2);
                    insideD = isinSet(iDeq,xout{end}(ttemp).',xsym,tsym);
                    if insideD
                        break
                    end                    
                end
                if insideD == 0
                    break;
                elseif insideD == 1
                    [j tout jout xout] = jump(g,j,tout,jout,xout,tsym);
                end
            end
        case 2 % flow priority (solve only one jump)
            insideD = 0;
            for irDeq = 1:rDeq % check for all disconnected sets
                iDeq = Deq(irDeq,:);
                ttemp = tout(end,2);            
                insideD = isinSet(iDeq,xout{end}(ttemp).',xsym,tsym);                            
                if insideD
                    break
                end
            end
            if insideD == 1
                [j tout jout xout] = jump(g,j,tout,jout,xout,tsym); 
            end
            for irCeq = 1:rCeq % check for all disconnected sets
                iCeq = Ceq(irCeq,:);
                ttemp = tout(end,2);
                insideC = isinSet(iCeq,xout{end}(ttemp).',xsym,tsym);
                if insideC
                    break
                end
            end
            if insideD == 0 && insideC == 0 
                break;
            end
    end
    if tout(end,2) == inf
        fprintf('Completed: %3.0f%% thd [time: %3.5f, jumps: %3.0f ]\n',...
            100,double(ttemp),j);
        warning('The last interval ends up to infinity, complete solution was found.');
    else
        fprintf('Completed: %3.0f%% thd [time: %3.5f, jumps: %3.0f ]\n',...
            max(100*j/JSPAN(end),eval(100*subs(tout(end,2))/TSPAN(end))),double(ttemp),j);
    end
end
%% Save outputs
t = tout;
x = xout;
j = jout;
fprintf('\nDone :p\n');
end

function [Ij,IjLeft,IjRight,IjEq] = zeroevents(Seq,xsol,xt,tsym,tini)
global vpaPr
xfields = fieldnames(xsol); % get the 'names' of the variables
% set the symbolic varialbes from xsym
for ixt = 1:length(xt)
    syms(char(xt(ixt)));
end
eqi = 1;
for ieq = Seq % Replce the solutions in the ith disconnected set's equations
    eqIj(eqi) = ieq;
    for xi = 1:length(xt)
        ix = getfield(xsol, xfields{xi});
        eqIj(eqi) = subs(eqIj(eqi),...
            eval(xfields{xi}),ix);
    end
    eqi = eqi+1;
end
IjEq = Seq;
try
    warning('off','symbolic:solve:TooManyVars')
    Ij = vpa(solve(vpa(eqIj,vpaPr),tsym>=tini,tsym,'IgnoreAnalyticConstraints', true),vpaPr); % Solve the equations and get the intervals
    warning('on','symbolic:solve:TooManyVars')
    IjLeft  = vpa(feval(symengine,'Dom::Interval::left' ,Ij),vpaPr);
    IjRight = vpa(feval(symengine,'Dom::Interval::right',Ij),vpaPr);
catch
    if isnumeric(eval(Ij))
        warning('HyEQsymb:NumericCall','During symbolic solution: Numeric solver has been called (try change sets C and D to avoid this).')        
        IjLeft  = Ij;
        IjRight = Ij;
    else
        warning('HyEQsymb:AnalyticConsIgnored','During symbolic solution: Analytic Constraints were ignored.')
        warning('off','symbolic:solve:TooManyVars')
        Ij = solve(vpa(eqIj,vpaPr),'IgnoreAnalyticConstraints', true); % Solve the equations and get the intervals
        warning('on','symbolic:solve:TooManyVars')
        IjLeft  = vpa(feval(symengine,'Dom::Interval::left' ,Ij),vpaPr);
        IjRight = vpa(feval(symengine,'Dom::Interval::right',Ij),vpaPr);
    end
end

end

function [j tout jout xout] = jump(g,j,tout,jout,xout,tsym)
% Jump
j = j+1;
ttemp = tout(end,2);
y = g(xout{end}(ttemp).');
% Save results for jumps
tout(end+1,1) = tout(end,2);
tout(end,2) = tout(end,1);
xout{end+1} = symfun(y.',tsym);
jout(end+1) = j;
end
