function [x_sliced,t_sliced] = plotHarcColor3D(t,j,x,L,jstar,modif_in)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matlab M-file       Project: HyEQ Toolbox  @ Hybrid Dynamics and Control
% Lab, http://www.u.arizona.edu/~sricardo/index.php?n=Main.Software
%
% Filename: plotHarcColor3D.m
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plotHarcColor3D   Hybrid arc plot in 3 dimensions with color.
%   plotHarcColor3D(t,j,x,L) plots hybrid time vector (t,j) versus the 3D
%   vector x taking into account jumps j. The hybrid arc is plotted with L
%   data as color. The input vectors t,j, x, L must have the same length
%   and x must have three columns.
%
%   plotHarcColor3D(t,j,x,L,jstar) If a specific interval in j is required,
%   jstar = [j-initial j-final] must be provided.
%
%   plotHarcColor3D(t,j,x,L,jstar,modif_in) Modificator must be a cell
%   array that contains the standard matlab ploting modificators (see
%   example)
%
%   Various line types, plot symbols and colors may be obtained with
%   plotHarc(t,j,x,jstar,modificator) where modificator is a cell array
%   created with the following strings:
%
%          b     blue          .     point              -     solid
%          g     green         o     circle             :     dotted
%          r     red           x     x-mark             -.    dashdot 
%          c     cyan          +     plus               --    dashed   
%          m     magenta       *     star             (none)  no line
%          y     yellow        s     square
%          k     black         d     diamond
%          w     white         v     triangle (down)
%                              ^     triangle (up)
%                              <     triangle (left)
%                              >     triangle (right)
%                              p     pentagram
%                              h     hexagram
%
%   Example
%         % Load a hybrid arc
% 
%         load Data_Example_1_2_BB
%         
%         % Compute a test function
%         L = x(:,2).^2*1/2+9.81.*x(:,1);
% 
%         % Plot the hybrid arc Vs the hybrid time domain (t,j)
%         figure(1)
%         plotHarcColor3D(t,j,[t,x],L);
%         view(3)
%         xlabel('time')
%         ylabel('x_1')
%         zlabel('x_2')
%         grid on
% 
% 
%         % Plot the hybrid arc Vs the hybrid time domain (t,j) for a specified jump span $j\in[3,5]$
% 
%         figure(2)
%         plotHarcColor3D(t,j,[x,L],L,[3,5]);
%         view(3)
%         xlabel('x_1')
%         ylabel('x_2')
%         zlabel('L')
%         grid on
% 
%         % Plot and get the hybrid arc Vs the hybrid time domain (t,j) for a specified jump
% 
%         figure(3)
%         [x_sliced,t_sliced] = plotHarcColor3D(t,j,[x,L],L,[5]);
%         view(3)
%         xlabel('x_1')
%         ylabel('x_2')
%         zlabel('L')
%         grid on
% 
% 
%         % Use the modificators
%         figure(4)
% 
%         modif_in{1} = 'LineWidth';
%         modif_in{2} = 2;
% 
%         plotHarcColor3D(t,j,[x,L],L,[],modif_in);
%         view(3)
%         xlabel('x_1')
%         ylabel('x_2')
%         zlabel('L')
%         hold on
%         grid on
% 
%         % Plot a phase plane e.g., $x_1$ Vs $x_2$
% 
%         figure(5)
%         plotHarcColor3D(x(:,1),j,[x,L],L);
%         view(3)
%         xlabel('x_1')
%         ylabel('x_2')
%         zlabel('L')
%         xlim([-0.1, 1])
%         grid on
%
%   See also plotflows, plotHarc, plotHarcColor, plotHarcColor3D,
%   plotHybridArc, plotjumps.
%
%   Copyright @ Hybrid Dynamics and Control Lab,
%   http://www.u.arizona.edu/~sricardo/index.php?n=Main.Software $Revision:
%   0.0.0.1 $  $Date: 2014/01/07 10:48:24 $

switch nargin
    case {1,2,3}
        fprintf('Please use plotHarc instead, too few inputs')
        return
    case 4
        iini = j(1);
        ifini = j(end);
    case {5,6}
        if isempty(jstar)
            iini = 0;
            ifini = j(end);
        else
            iini = jstar(1);
            try
                ifini = jstar(2);
            catch
                ifini = jstar(1);
            end
        end
end
if ~exist('modif_in','var')
    modif_in{1} = 'LineWidth';
    modif_in{2} = 1;
end
[nt,nin] = size(x);
switch nin
    case {3}
    otherwise
        fprintf('Please use plotHarc instead, x does not have the proper size')
        return
end

modificator{1} = '*';
modificator{2} = '--';

x_sliced{ifini-iini+1} = [];
t_sliced{ifini-iini+1} = [];
col{ifini-iini+1} = [];
for ij=1:ifini-iini+1
    iij = find(j(:)==iini+ij-1,1,'first');
    fij = find(j(:)==iini+ij-1,1,'last');
    x_sliced{ij} = x(iij:fij,:);
    t_sliced{ij} = t(iij:fij);
    col{ij}      = L(iij:fij);
    %     z = zeros(size(t_sliced{ij}));
    if fij-iij>0
        switch nin
            case 3
                color_line3HyEQ(x_sliced{ij}(:,1),...
                    x_sliced{ij}(:,2),...
                    x_sliced{ij}(:,3),...
                    col{ij},modif_in);
        end
    else
        color_line3HyEQ(x_sliced{ij}(:,1)*ones(2,1),...
            x_sliced{ij}(:,2)*ones(2,1),...
            x_sliced{ij}(:,3)*ones(2,1),...
            col{ij}*ones(2,1),modif_in);
    end
    hold on;
    if ij>1
        %         z = zeros(size(t(iij-1:iij)));
        color_line3HyEQ(...
            [x(iij-1:iij,1);x(iij-1:iij,1)],...
            [x(iij-1:iij,2);x(iij-1:iij,2)],...
            [x(iij-1:iij,3);x(iij-1:iij,3)],...
            [L(iij-1:iij);L(iij-1:iij)],...
            modificator);
        
    end
end
hold off;
